@@ -1,3 +1,22 @@
+(* Co-installability tools
+ * http://coinst.irill.org/
+ * Copyright (C) 2005-2011 Jérôme Vouillon
+ * Laboratoire PPS - CNRS Université Paris Diderot
+ *
+ * These programs are free software; you can redistribute them and/or
+ * modify them under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ *)
+
 (*
 References
 ----------
@@ -166,8 +185,8 @@ let tags =
    (1082, (STRING_ARRAY, "CHANGELOGTEXT", false));
    (1085, (STRING,       "PREINPROG", false));
    (1086, (STRING,       "POSTINPROG", false));
-(*   (1087, (STRING,       "PREUNPROG", false));*)
-   (1087, (STRING_ARRAY, "PREUNPROG", false));
+   (1087, (STRING,       "PREUNPROG", false));
+(*   (1087, (STRING_ARRAY, "PREUNPROG", false));*)
    (1088, (STRING,       "POSTUNPROG", false));
    (1090, (STRING_ARRAY, "OBSOLETENAME", false));
    (1091, (STRING,       "VERIFYSCRIPTPROG", false));
@@ -288,7 +307,9 @@ let _BASENAMES = 1117
 let _DIRNAMES = 1118
 let _DISTEPOCH = 1218
 
-let etag entry i = let (tag, _, _, _) = entry.(i) in tag
+let etag entry i =
+  if i >= Array.length entry then max_int else
+  let (tag, _, _, _) = entry.(i) in tag
 
 let rec move_to entry i tag =
   if etag entry i >= tag then i else move_to entry (i + 1) tag
@@ -361,7 +382,7 @@ let intern_file filemodes filemd5s filelinktos i =
   | 0o120000 -> Link (filelinktos.(i))
   | 0o140000 -> Sock
   | 0o10000  -> Pipe
-  | 0o100000 -> Reg (filemd5s.(i))
+  | 0o100000 -> Reg (String.sub filemd5s.(i) 0 32)
   | _        -> Util.fail (Format.sprintf "unknown mode %o" mode)
 
 let pr_info ch i =
@@ -496,9 +517,10 @@ let parse_version s =
   if s = "" then
     None
   else if not (Str.string_match version_re_1 s 0 ||
-               Str.string_match version_re_2 s 0) then
-    failwith ("Bad version " ^ s)
-  else begin
+               Str.string_match version_re_2 s 0) then begin
+    Util.print_warning (Format.sprintf "bad version '%s'" s);
+    raise Skip
+  end else begin
     let epoch =
       try
         let s = Str.matched_group 2 s in
@@ -738,10 +760,14 @@ let parse_header pool ignored_packages ch =
       if file_info then eint32_array store entry (i + 2) _FILEFLAGS else [||]
     in
     let i = move_to entry i _PROVIDENAME in
-    let providename = estring_array store entry i _PROVIDENAME in
-    let requireflags = eint32_array store entry (i + 1) _REQUIREFLAGS in
-    let requirename = estring_array store entry (i + 2) _REQUIRENAME in
-    let requireversion = estring_array store entry (i + 3) _REQUIREVERSION in
+    let has_provides = etag entry i = _PROVIDENAME in
+    let providename =
+      if has_provides then estring_array store entry i _PROVIDENAME else [||]
+    in
+    let i = move_to entry i _REQUIREFLAGS in
+    let requireflags = eint32_array store entry i _REQUIREFLAGS in
+    let requirename = estring_array store entry (i + 1) _REQUIRENAME in
+    let requireversion = estring_array store entry (i + 2) _REQUIREVERSION in
     let i = move_to entry i _CONFLICTFLAGS in
     let has_confl = etag entry i = _CONFLICTFLAGS in
     let conflictflags =
@@ -754,8 +780,13 @@ let parse_header pool ignored_packages ch =
       else [||]
     in
     let i = move_to entry i _PROVIDEFLAGS in
-    let provideflags = eint32_array store entry i _PROVIDEFLAGS in
-    let provideversion = estring_array store entry (i + 1) _PROVIDEVERSION in
+    let provideflags =
+      if has_provides then eint32_array store entry i _PROVIDEFLAGS else [||]
+    in
+    let provideversion =
+      if has_provides then estring_array store entry (i + 1) _PROVIDEVERSION
+      else [||]
+    in
     let i = move_to entry i _DIRINDEXES in
     let non_empty = etag entry i = _DIRINDEXES in
     let dirindexes =
